;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite Respawner
;; by Sonikku
;; Description: A sprite that you place at the same X/Y positions as another sprite. It aims to respawn the sprite whenever
;; the target sprite is killed or otherwise despawns.
;; This sprite is intended to be used with a sprite such as the Magikoopa disassembly, Lakitu disassembly, or the Boss Bass,
;; but can realistically work with most others.
;;
;; If the sprite is turned into a coin (i.e. Silver POW, Fire Flower), it'll only respawn when that coin is collected or despawns.
;;
;; I only offer support on this sprite if it does not function as intended for most common sprites. Less common sprites I will
;; not offer support on, so adapt this sprite to your own sprites as you see fit.
;; 
;; Uses Extra Bit: YES
;; If Extra Bit is set, it will respawn the sprite in the same X and Y positions as where it originally was.
;; If Extra Bit is clear, it will respawn the sprite in the same Y position, but from a random side of the screen.
;;
;; Extra Property Byte 1 controls the timer before the sprite respawns
;; 
;; Extra Property Byte 2:
;; Bit 0 (01): Don't spawn if Mario is too close (when Extra Bit is set)
;; Bit 1 (02): Display smoke effect on-generate (when Extra Bit is set)
;; Bit 2 (04): Play sound effect on-generate
;; Bit 3 (08): Use free RAM to disable respawning sprites
;; Bit 4 (10): Suppress sound effect if Extra Bit is clear

!SmokeType	= $01		; Smoke Type
!SmokeTimer	= $1B		; Smoke Timer
				;; See this link for details: https://www.smwcentral.net/?p=nmap&m=smwram#7E17C0

!Sound		= $03		; Sound Effect Number
!SoundBank	= $1DFC		; Sound Effect Bank (Possible values are: $1DF9, $1DFA, $1DFB)
				;; See this link for details: https://www.smwcentral.net/?p=viewthread&t=6665

!FreeRAM	= $79		; RAM address to use for "should not respawn" flag
				;; Explanation: if this flag is enabled to be used, you can create additional code
				;; (i.e. generators, levelasm) to make this sprite temporarily or permanently stop functioning based
				;; whatever criteria you program in. One idea is making it based on screen number.
				;; If this flag is non-zero and positive, it'll cause this sprite to stop functioning.
				;; If this flag is non-zero and negative, it'll cause this sprite to kill itself.

print "INIT ",pc
	LDA #$01
	STA $C2,x
	RTL
print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Sprite_Main
	PLB
	RTL
Sprite_Main:
	LDA $7FAB34,x	; \
	AND #$08	;  | branch if Extra Property Byte 2 (bit 3) is set
	BEQ +		; /
	LDA !FreeRAM	; \ branch if free ram is clear
	BEQ +		; /
	BPL ++		; branch if free ram is negative
	STZ $14C8,x	; kill sprite
++	RTS		; 
+	LDA $C2,x	; 
	BNE +		; 
	JMP .dont_checkforsprite
+	LDY #$0B	; number of times to loop
-	STX $00		; \ don't run if sprite checked is self
	CPY $00		;  |
	BEQ .loop	; /
	LDA $14C8,x	; \
	CMP #$01	;  | run if sprite is init
	BEQ +		; /
	CMP #$0C	; \ don't run if sprite is level-end powerup
	BEQ .loop	; /
	CMP #$08	; \ don't run if sprite is dead
	BCC .loop	; /
+	JSL $03B69F	; \
	PHX		;  | 
	TYX		;  | 
	JSL $03B6E5	;  | run if sprite contacts this one
	PLX		;  | 
	JSL $03B72B	;  | 
	BCS .touchsprite; /
.loop	DEY		; 
	BPL -		; 
	RTS		; 
.touchsprite
	TYA		; \ set index of obtained sprite
	STA $1602,x	; /

	LDA $00E4,y	; \
	STA $E4,x	;  | 
	LDA $14E0,y	;  | 
	STA $14E0,x	;  | 
	LDA $00D8,y	;  | set up x/y positions for later if needed
	STA $D8,x	;  | 
	LDA $14D4,y	;  | 
	STA $14D4,x	; /

	LDA $14C8,y	; \ save sprite state
	STA $1528,x	; /

	STZ $C2,x	; sprite state = ready to respawn

	PHX		; \
	TYX		;  | 
	LDA $7FAB10,x	;  | 
	AND #$08	;  | 
	BEQ .normalspr	;  | 
	STZ $00		;  | 
	LDA $7FAB10,x	;  | get custom sprite number
	AND #$04	;  | 
	BEQ +		;  | 
	INC $00		;  | 
+	LDA $7FAB9E,x	;  | 
	PLX		;  | 
	STA $1626,x	; /
	LDA #$01	; \ mark as custom sprite
	STA $1504,x	; /
	LDA $00		; \ get extra bit setting
	STA $1594,x	; /
	BRA +

.normalspr
	PLX		;  | 
	LDA $009E,y	; \ get regular sprite number
	STA $1626,x	; /
+	LDA #$FF	; \ force target sprite to not respawn automatically
	STA $161A,y	; /
	RTS		; 

.dont_checkforsprite
	LDA $160E,x	; \ branch if sprite ready to respawn
	BNE .respawn_spr; /
	LDY $1602,x	; \
	LDA $14C8,y	;  | 
	CMP #$0C	;  | check if sprite is alive
	BEQ .respawn	;  | 
	CMP #$08	;  | 
	BCS .return	; /
.respawn
	LDA $7FAB28,x	; \ set timer before spawning sprite
	STA $1540,x	; /
	LDA #$01	; \ sprite ready to respawn
	STA $160E,x	; /
.return	RTS		; 

.respawn_spr
	LDA $1540,x	; \ branch if not ready to spawn
	BNE .return	; /

	LDA $7FAB10,x	; \
	AND #$04	;  | skip if extra bit clear
	BEQ +		; /
	LDA $14E0,x	; \
	XBA		;  | 
	LDA $E4,x	;  | 
	REP #$20	;  | 
	SEC		;  | 
	SBC $1A		;  | branch if within screen borders
	CLC		;  | 
	ADC #$0020	;  | 
	CMP #$0140	;  | 
	SEP #$20	;  | 
	BCS .return	; /

+	JSL $02A9DE	; \ branch if no free slots
	BMI .return	; /
	LDA $7FAB10,x	; \
	AND #$04	;  | branch if extra bit is clear
	BEQ +		; /
	LDA $7FAB34,x	; \
	AND #$01	;  | branch if Extra Property Byte 2 (bit 0) is set
	BEQ +		; /
	LDA $94		; \
	SEC		;  | 
	SBC $E4,x	;  | 
	CLC		;  | branch if mario is too close
	ADC #$11	;  | 
	CMP #$22	;  | 
	BCC .return	; /
+	STZ $160E,x	; don't spawn more
	TYA		; \ set index of new sprite
	STA $1602,x	; /

	LDA $D8,x	; \
	STA $00D8,y	;  | set y position
	LDA $14D4,x	;  | 
	STA $14D4,y	; /

	LDA $7FAB10,x	; \
	AND #$04	;  | branch if extra bit set
	BNE .extra_set	; /

	JSL $01ACF9	; \
	AND #$01	;  | 
	PHX		;  | 
	TAX		;  | 
	LDA $1A		;  | 
	CLC		;  | sprite x position is randomly assigned to either left or right of screen
	ADC $02B1B8,x	;  | 
	STA $00E4,y	;  | 
	LDA $1B		;  | 
	ADC $02B1BA,X	;  | 
	STA $14E0,y	;  | 
	PLX		; /
	BRA +		; 
.extra_set
	LDA $E4,x	; \
	STA $00E4,y	;  | sprite x position is assigned to original x position
	LDA $14E0,x	;  | 
	STA $14E0,y	; /
+
	LDA $1528,x	; \
	CMP #$08	;  | branch if sprite state was #$08, and if so make it init
	BEQ .is_init	; /
	CMP #$09	; \
	BCC +		;  | make sprite carriable if original was carried/carriable/kicked
	LDA #$09	;  | 
	BRA +		; /
.is_init
	LDA #$01	; sprite state = init
+	STA $14C8,y	; 
	LDA $1626,x	; \
	CMP #$DA	;  | 
	BCC +		;  | if sprite number is above #$DA, subtract it (aka shells)
	SEC		;  | 
	SBC #$D6	;  | 
+	STA $009E,y	; /
	LDA $1626,x	; \
	PHX		;  | 
	TYX		;  | set custom sprite number and init
	STA $7FAB9E,x	;  | (custom sprite number isn't used in regular sprites, and this can be a good way to track the relative sprite number)
	JSL $07F7D2	;  | 
	PLX		; /

	LDA $1594,x	; \ get extra bit setting
	STA $0C		; /

	LDA $1504,x	; \
	PHX		;  |
	BEQ .not_custom	;  | 
	TYX		;  | set up init if sprite = custom
	JSL $0187A7	;  | 
	LDA #$08	;  | 
	PHY		;  | 
	LDY $0C		;  | 
	BEQ +		;  | 
	ORA #$04	;  | 
+	PLY		;  | factor in extra bit for new sprite 
	STA $7FAB10,x	;  | 
.not_custom
	PLX		; /

	STZ $00		; \
	LDA $94		;  | 
	SEC		;  | 
	SBC $00E4,y	;  | 
	LDA $95		;  | set up sprite direction to face mario
	SBC $14E0,y	;  | 
	BPL +		;  | 
	INC $00		;  | 
+	LDA $00		;  | 
	STA $157C,y	; /

	LDA $1504,x	; \
	PHX		;  | 
	BNE .return2	; /
	LDA $00		; \
	TAX		;  | 
	LDA $009E,y	;  | 
	CMP #$1C	;  | add exception if sprite is bullet bill
	BNE +		;  | 
	TXA		;  | 
	STA $00C2,y	;  | 
	BRA .return2	; /
+	CMP #$35	; \ 
	BNE .return2	;  | add exception if sprite is yoshi
	LDA #$08	;  | 
	STA $14C8,y	; /
.return2
	PLX		; 

	LDA $7FAB10,x	; \
	AND #$04	;  | branch if extra bit is clear
	BEQ .checkforsnd; /
	LDA $7FAB34,x	; \
	AND #$02	;  | branch if Extra Property Byte 2 (bit 1) is set
	BEQ .checkforsnd; /
	LDY #$03	; times to loop
-	LDA $17C0,y	; \ branch if slot not free
	BNE +		; /
	LDA #!SmokeType	; \ set smoke type
	STA $17C0,y	; /
	LDA $E4,x	; \
	STA $17C8,y	;  | set smoke coordinates
	LDA $D8,x	;  | 
	STA $17C4,y	; /
	LDA #!SmokeTimer; \ set smoke timer
	STA $17CC,y	; /
+	DEY		; 
	BPL -		; 
.checkforsnd
	LDA $7FAB34,x	; \
	AND #$04	;  | branch if Extra Property Byte 2 (bit 2) is set
	BEQ +		; /
	LDA $7FAB34,x	; \
	AND #$10	;  | branch if Extra Property Byte 2 (bit 5) is clear
	BEQ .playsnd	; /
	LDA $7FAB10,x	; \
	AND #$04	;  | branch if Extra Bit is cleared
	BEQ +		; /
.playsnd
	LDA #!Sound	; \ play sound
	STA !SoundBank	; /
+	RTS		; 

.xpos_lo
	db $00,$F0
.xpos_hi
	db $01,$00