;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Timer, by imamelia
;;
;; This sprite creates a timer on the status bar.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the timer starts at a certain
;; value and ticks down until it reaches zero.  If the first
;; extra bit is set, the timer starts at zero and goes up until
;; it reaches 59:59(:59), at which point it resets to 00:00(:00).
;;
;; Extra Property Byte 1:
;;
;; Bit 0: show frames (if this bit is set, the timer will show only
;;	minutes and seconds; if it is clear, the timer will show
;;	frames as well)
;; Bit 1: show the clock icon (if this bit is clear, the clock icon will
;;	be there; if it is set, the clock will not be there)
;;
;; NOTE: When the timer is going down, the initial time it has is
;; determined by the initial position of the sprite.  The X position
;; sets the minutes, and the Y position sets the seconds.
;; The Y position also can cause the timer to kill the player,
;; teleport him/her to the screen exit of screen 00, or set the blue
;; (or silver) P-switch timer.  (Place the sprite within an odd
;; subscreen for these effects...or you can edit the pointer table
;; to set it up however you want.)
;;
;; NOTE 2: This sprite requires 4 bytes of empty RAM.  The default is
;; $0DC3-0DC5 plus $7C, but that can be changed.  (In the case of $7C,
;; you may want to make sure that it is an address that clears on level
;; load.)
;;
;; NOTE 3: This sprite doesn't work very well with secondary
;; entrances.  It won't crash or anything like that, but it will
;; reset or fail to show up.  I'd advise against putting
;; any secondary entrances in any level that contains one of these,
;; although just plain screen exits are okay as long as you don't
;; mind the timer not appearing in the next level.
;;
;; Inspired by, but not based off of, Iceguy's timer sprites.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Minutes = $0DC3	; the RAM address used for the minutes
!Seconds = $0DC4	; the RAM address used for the seconds
!Frames = $0DC5		; the RAM address used for the frames
!Position = $0F09	; the timer's position on the status bar
!SwitchTimer = $B0	; the timer to set for the blue P-switch
!EffectFlag = $7C	; if this flag is set, the effects have already been done

InitialMinutes:	; the number of minutes for each possible X position (0-15, but that can be changed)

db $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F

InitialSeconds:	; the number of seconds for each possible Y position
		; (0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 0, 15, 30, 45)

db $00,$05,$0A,$0F,$14,$19,$1E,$23,$28,$2D,$32,$37,$00,$0F,$1E,$2D

Effects:	; the effects for each possible Y position (plus X-high byte)...you can even make your own effects!

dw NoEffect,NoEffect,NoEffect,NoEffect	; do nothing if 0-3
dw NoEffect,NoEffect,NoEffect,NoEffect	; do nothing if 4-7
dw NoEffect,NoEffect,NoEffect,NoEffect	; do nothing if 8-B
dw Switch,Switch,Switch,Switch	; set the blue P-switch timer if C-F
dw Teleport,Teleport,Teleport,Teleport	; teleport the player if 10-13
dw Teleport,Teleport,Teleport,Teleport	; teleport the player if 14-17
dw Teleport,Teleport,Teleport,Teleport	; teleport the player if 18-1B
dw Death,Death,Death,Death		; kill the player if 1C-1F

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
PHB
PHK
PLB
LDA $7FAB10,x	; check the extra bits
AND #$04	; the extra bit
STA $C2,x	; set the sprite state (increasing or decreasing)
JSR SetInitialTime	; set the starting amount of time that should be on the timer
JSR SetEffectsIndex	; use the sprite X/Y position to get a value for the effects subroutine
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
		JSR StatusBarShow	;this is here so if freeze
					;is active, won't be overwritten
					;by other tiles (stores every frame).
	LDA $9D				;if lock animation 
	BNE lock_ani			;is on, then freeze timer
		JSR SubOffscreen
		JSR SetEffects
		JSR SetTime
lock_ani:
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; set initial time subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetInitialTime:

LDA $C2,x		; check sprite state
BNE SetToZero		; if the timer is increasing, then just set it to 00:00(:00)

LDA $E4,x		; sprite initial X position (low byte)
LSR
LSR
LSR
LSR			; divide by 16 to get it in terms of tiles
TAY			; into Y
LDA InitialMinutes,y		; set the starting number of minutes depending on this value
STA !Minutes		;

LDA $D8,x		; sprite initial Y position (low byte)
LSR
LSR
LSR
LSR			; divide by 16 to get it in terms of tiles
TAY			; into Y
LDA InitialSeconds,y	; set the starting number of seconds depending on this value
STA !Seconds		;

LDA #$01
STA !Frames

RTS

SetToZero:

STZ !Minutes		; 00 minutes
STZ !Seconds		; 00 seconds
STZ !Frames

RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; set effects index subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetEffectsIndex:

LDA $D8,x	; sprite Y position low byte
LSR
LSR
LSR
LSR		; divide by 16 to get it in terms of tiles
STA $00		; temporarily preserve

LDA $14E0,x	; sprite X position high byte
AND #$01	; we want only one bit
ASL
ASL
ASL
ASL		; shift left/multiply by 16
ORA $00		; and finally, transfer the resulting value
STA $1528,x	; to a sprite table

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutine to set the time
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetTime:

LDY #$3B		; 59 is the number we will use to reset "things"

LDA $C2,x	; if the timer is supposed to increment...
BNE Increasing	; use a different subroutine

LDA !Frames	; if frames,
ORA !Seconds	; seconds,
ORA !Minutes	; and minutes are all zero...
BEQ NoDec	; don't decrement anything

LDA !Frames	; if frames are 00...
BEQ ResetFrames	; then reset them to 59, but if not...
DEC !Frames	; simply decrement them
RTS		; and return

ResetFrames:	;

STY !Frames	; set frames to 59

LDA !Seconds	; if seconds are 00...
BEQ ResetSeconds	; then reset them to 59 as well, but if not...
DEC !Seconds	; simply decrement them
RTS		; and return

ResetSeconds:	;

STY !Seconds	; set seconds to 59

DEC !Minutes	; and decrement minutes (the minutes can't be zero, because we already checked that)

NoDec:

RTS

Increasing:

INY		; Y = 3C (60) so we can use it to compare

INC !Frames	; increment frames
CPY !Frames	; if frames are 60...
BEQ ResetFrames2	; then reset them to 00
RTS

ResetFrames2:	;

STZ !Frames	; reset frames to 00
INC !Seconds	; increment seconds
CPY !Seconds	; if seconds are 60...
BEQ ResetSeconds2	; then reset them to 00
RTS

ResetSeconds2:		;
STZ !Seconds		; reset seconds to 00
INC !Minutes		; increment minutes
CPY !Minutes		; if minutes are 60...
BNE NoResetMinutes2	; then reset them to 00
STZ !Minutes		;
NoResetMinutes2:
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; display time on status bar subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

StatusBarShow:

LDA $7FAB28,x	; extra property byte 1
AND #$02	; if bit 1 is set...
BNE NoShowClock	; don't show the clock

LDA #$76		; 76 = the clock tile
STA !Position	; store to the first status bar slot

NoShowClock:

LDA !Minutes	; first, store minutes
JSR HexToDec	; hexadecimal-to-decimal conversion subroutine
STY !Position+1	; store the tens digit of the minutes
STA !Position+2	; store the ones digit of the minutes

LDA !Seconds	; next, store seconds
JSR HexToDec	; hexadecimal-to-decimal conversion subroutine
STY !Position+4	; store the tens digit of the seconds
STA !Position+5	; store the ones digit of the seconds

LDA $7FAB28,x	; extra property byte 1

PHX		; preserve the sprite index
LDX #$78		; 78 is the colon tile

AND #$01	; if bit 0 of the extra property byte is set...
BNE NoShowFrames	; don't show the frames

LDA !Frames	; finally, store frames
JSR HexToDec	; hexadecimal-to-decimal conversion subroutine
STY !Position+7	; store the tens digit of the frames
STA !Position+8	; store the ones digit of the frames
STX !Position+6	; display the second colon

NoShowFrames:	;

STX !Position+3	; display the first colon
PLX		; pull sprite index back

RTS		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; effects subroutine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SetEffects:

LDA !Frames	; if frames,
ORA !Seconds	; seconds,
ORA !Minutes	; and minutes are all zero...
BNE NoEffect	; jump to effects subroutine

LDA $C2,x	; if the sprite is supposed to start at zero and increment...
BNE NoEffect	; don't set any effects

LDA $1528,x		; load effects index
JSR ExecutePointer16Bit	; jump to pointer subroutine

NoEffect:

RTS

Teleport:		; the teleport player subroutine

LDY $95		; the current screen

LDA $19B8	; screen exit of screen 00, low byte
STA $19B8,y	; set the current screen exit to that value
LDA $19D8	; screen exit of screen 00, high byte
STA $19D8,y	; set the current screen exit to that value

LDA #$06		; set teleporting action
STA $71		;
STZ $88		;
STZ $89		;

RTS

Death:		;

LDA $7C		; if the effect flag is already set,
BNE NoSetEffect1	; don't activate the effect again

INC $7C		; set the effect flag
JSL $00F606	; jump to death subroutine

NoSetEffect1:

RTS

Switch:		;

LDA $7C		; don't reset the timer
BNE NoSetMusic	; if it has already been set

INC $7C		; set effect flag

LDA #!SwitchTimer	; set the blue P-switch timer
STA $14AD	; You can replace this with STA $14AE if you want the silver timer instead.

LDA $0DDA	;
BMI NoSetMusic	; check to see if the music is "abnormal"

LDA #$0E		;
STA $1DFB	; set the P-switch music

NoSetMusic:

RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; hexadecimal-to-decimal conversion subroutine (2 digits)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

HexToDec:                                    ;HEX->DEC converter
                     LDY #$00                ;> Set Y to 0
StartCompare:
                     CMP #$0A                ;\
                     BCC ReturnLong          ; |While A >= 10:
                     SBC #$0A                ; |Decrease A by 10
                     INY                     ; |Increase Y by 1
                     BRA StartCompare        ;/
ReturnLong:
                     RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 16-bit pointer subroutine (I designed it myself)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ExecutePointer16Bit:

PHX		; preserve X, because there is a very likely chance that it is already being used
ASL A		; multiply the base value to get the desired index
TAX		; put the resulting value into X
REP #$20		; set A to 16-bit
LDA Effects,x	; load value from pointer table
STA $0A		; and store that to scratch RAM
SEP #$20		; set A back to 8-bit
PLX		; pull X register back
JMP ($000A)	; jump to address that was stored

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

Table3:              db $40,$B0
Table6:              db $01,$FF 
Table4:              db $30,$C0,$A0,$80,$A0,$40,$60,$B0 
Table5:              db $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffscreen:   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA Store03            ;  | 
SubOffscreenX1:   LDA #$04                ;  |
                    BRA Store03            ;  |
SubOffscreenX2:   LDA #$02                ;  |
Store03:       
	    STA $03                 ;  |
                    BRA StartSub           ;  |
SubOffscreenX3:   STZ $03                 ; /

StartSub:           JSR SubIsOffscreen   ; \ if sprite is not off screen, return
                    BEQ Return2            ; /    
                    LDA $5B                 ; \  goto VerticalLevel if vertical level
                    AND #$01                ;  |
                    BNE VerticalLevel      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL EraseSprite        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;
                    CLC                     ;
                    ADC Table4,y            ;
                    ROL $00                 ;
                    CMP $E4,x               ;x pos ;
                    PHP                     ;
                    LDA $1B                 ;x boundry hi ;
                    LSR $00                 ;
                    ADC Table5,y            ;
                    PLP                     ;
                    SBC $14E0,x             ;x pos high
                    STA $00                 ;
                    LSR $01                 ;
                    BCC Label20             ;
                    EOR #$80                ;
                    STA $00                 ;
Label20:             LDA $00                 ;
                    BPL Return2            ;
EraseSprite:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KillSprite         ; /
                    LDY $161A,x             ;
                    CPY #$FF                ;
                    BEQ KillSprite        ;
                    LDA #$00                ; \ 
                    STA $1938,y             ; /     
KillSprite:         STZ $14C8,x             ; erase sprite
Return2:            RTS                     ; return

VerticalLevel:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE Return2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS Return2            ; /
                    AND #$01                ;
                    STA $01                 ;
                    TAY                     ;
                    LDA $1C                 ;
                    CLC                     ;
                    ADC Table3,y            ;
                    ROL $00                 ;
                    CMP $D8,x               ;
                    PHP                     ;
                    LDA $001D             ;
                    LSR $00                 ;
                    ADC Table6,y            ;
                    PLP                     ;
                    SBC $14D4,x             ;
                    STA $00                 ;
                    LDY $01                 ;
                    BEQ Label22             ;
                    EOR #$80                ;
                    STA $00                 ;
Label22:             LDA $00                 ;
                    BPL Return2            ;
                    BMI EraseSprite        ;

SubIsOffscreen:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return






